import random

from django.db import models
from django.db.models import TextChoices
from django.core.validators import MinLengthValidator

from core.settings import faker


class StatusChoices(TextChoices):
    ACCEPTED = 'accepted', 'Разрешено'
    UNACCEPTED = 'unaccepted', 'Не разрешено'
    CHECK = 'check', 'На проверке'


class ArticlesManager(models.Manager):
    def all(self, *args, **kwargs):
        # Article.objects.all() -> Article.objects.filter(status='accepted')
        return super().all().filter(status='accepted')
        # Article.objects.all().filter(status='accepted')

    def create_random_tags(self):
        from web.models import Tag

        random_count = random.randint(5, 15)

        for obj in self.get_queryset():
            obj.tags.add(
                *(
                    Tag(name=faker.company()).save()
                    for _ in range(random_count)
                )
            )

    def create_random_comments(self):
        from web.models import Comment

        random_count = random.randint(5, 15)

        for obj in self.get_queryset():
            obj.comments.bulk_create(
                (
                    Comment(text=faker.text(), article_id=obj.id)
                    for _ in range(random_count)
                )
            )


class Article(models.Model):
    objects = ArticlesManager()
    title = models.CharField(
        max_length=200,
        null=False,  # <null>
        blank=False,  # ''
        verbose_name='Заголовок',
        validators=[
            MinLengthValidator(5)
        ]
    )

    tags = models.ManyToManyField('web.Tag', related_name='articles', blank=True)

    # accepted, check, unaccepted
    status = models.CharField(
        max_length=100,
        choices=StatusChoices.choices,
        default='check'
    )

    text = models.TextField(
        max_length=3000,
        null=False,
        blank=False,
        verbose_name='Текст'
    )

    author = models.ForeignKey(
        'accounts.User',
        on_delete=models.CASCADE,
        default=1,
        related_name='articles',
        verbose_name='Автор'
    )

    created_at = models.DateTimeField(
        auto_now_add=True,
        verbose_name='Дата создания'
    )

    updated_at = models.DateTimeField(
        auto_now=True,
        verbose_name='Дата обновления'
    )

    class Meta:
        verbose_name = 'статью'
        verbose_name_plural = 'статьи'
        permissions = [
            ('can_eat_pizza', 'Может съесть пиццу')
        ]

    def __str__(self):
        return f'{self.author}: {self.title}'
